package a1_solutionUsingReflection;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.event.ComponentListener;
import java.awt.font.FontRenderContext;
import java.awt.font.LineBreakMeasurer;
import java.awt.font.TextAttribute;
import java.awt.font.TextLayout;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.IOException;
import java.text.AttributedCharacterIterator;
import java.text.AttributedString;
import java.util.EventListener;
import java.util.Hashtable;
import java.util.List;
import java.util.Vector;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;

/**
 * A canvas which renders a smiley-face to the screen
 * 
 * Note: Canvas is a heavyweight superclass, which makes SmileyCanvas also
 * heavyweight. To convert this class to a lightweight, change "extends Canvas"
 * to "extends Component".
 * 
 * http://java.sun.com/products/jfc/tsc/articles/painting/src/PaintDemo.java
 */
public class A1Canvas extends Canvas {

	private A1ModelVariant2 a1Model;
	private LineBreakMeasurer lineMeasurer;

	// index of the first character in the paragraph.
	private int paragraphStart;

	// index of the first character after the end of the paragraph.
	private int paragraphEnd;

	public A1Canvas(A1ModelVariant2 a1Model) {
		this.a1Model = a1Model;
	}

	public Dimension getPreferredSize() {
		return new Dimension(600, 600);
	}

	/*
	 * Paint when the AWT tells us to...
	 */
	public void paint(Graphics g) {
		// Dynamically calculate size information
		// (the canvas may have been resized externally...)
		Dimension size = getSize();
		Graphics2D g2 = (Graphics2D) this.getGraphics();
		Image img = a1Model.getImage();
		img = img.getScaledInstance(size.width, size.height,
				Image.SCALE_AREA_AVERAGING);
		g2.drawImage(img, 0, 0, null);

		List<VisualMass> shapes = a1Model.getShapes();
		for (VisualMass vm : shapes) {
			g2.setColor(vm.getTheColour());
			g2.fill(vm.getCentreDot(size));
			g2.draw(vm.getOutlineForVisualMass(size));
		}

		drawStringAsSpecified(g2, a1Model.getTheText());

	}

	private void drawStringAsSpecified(Graphics2D g2d,
			AttributedString theString) {
		// body of this method derived directly from LineBreakSample.java
		// example from the Oracle website

		// Create a new LineBreakMeasurer from the paragraph.
		// It will be cached and re-used.
		if (lineMeasurer == null) {
			AttributedCharacterIterator paragraph = theString.getIterator();
			paragraphStart = paragraph.getBeginIndex();
			paragraphEnd = paragraph.getEndIndex();
			FontRenderContext frc = g2d.getFontRenderContext();
			lineMeasurer = new LineBreakMeasurer(paragraph, frc);
		}

		// Set break width to width of Component.
		float breakWidth = (float) getSize().width;
		// Set position to the index of the first character in the paragraph.
		lineMeasurer.setPosition(paragraphStart);

		// MB: we need to derive a list of lines in advance of their rendering;
		// we need to know the total number of lines in order to derive the
		// vertical anchor of the first line
		List<TextLayout> theLines = new Vector<TextLayout>();
		// Get lines until the entire paragraph has been displayed.
		while (lineMeasurer.getPosition() < paragraphEnd) {
			// Retrieve next layout. A cleverer program would also cache
			// these layouts until the component is re-sized.
			TextLayout layout = lineMeasurer.nextLayout(breakWidth);
			theLines.add(layout);
		}

		// MB: now we start the vertical anchor at the bottom-most position and
		// move upwards according to the number of lines to be drawn. We draw
		// the lines in reverse order.
		float drawPosY = (float) getSize().height;
		for (int i = theLines.size() - 1; i >= 0; i--) {
			TextLayout layout = theLines.get(i);
			// Compute pen x position. If the paragraph is right-to-left we
			// will align the TextLayouts to the right edge of the panel.
			// Note: this won't occur for the English text in this sample.
			// Note: drawPosX is always where the LEFT of the text is placed.
			float drawPosX = layout.isLeftToRight() ? 0 : breakWidth
					- layout.getAdvance();

			// Move y-coordinate by the ascent of the layout.
			drawPosY -= layout.getAscent();

			// Draw the TextLayout at (drawPosX, drawPosY).
			layout.draw(g2d, drawPosX, drawPosY);

			// Move y-coordinate in preparation for next layout.
			drawPosY -= layout.getDescent() + layout.getLeading();
		}
	}
}
